#! /usr/local/bin/perl -w

# 1315, Thu 6 Jan 00
#
# Nevil Brownlee, ITSS Technology Development, The University of Auckland

# Takes a .dif file and splits it into 'gmt days,'
#    i.e. one day's data starting at 0000 gmt
#          
# ./gmt_days.pl  from_file  to_file_header wk_start
#          e.g.  21dec.1600.nt_2.dif  data/nt_2.  6T19
#                wk_start default is 6T00, i.e. Sat 0000
#                6T19 (NZT) = 6T00 (PST)

use strict;
use Time::Local;

local %::month = (
      "Jan" =>  1, "Feb" =>  2, "Mar" =>  3, "Apr" =>  4,
      "May" =>  5, "Jun" =>  6, "Jul" =>  7, "Aug" =>  8,
      "Sep" =>  9, "Oct" => 10, "Nov" => 11, "Dec" => 12,
   );


my ($from_file, $line, @ntm_hdr);
my ($to_hdr, $to_trailer);

$from_file = shift;  # First item from command line
if (defined($from_file)) {
   open(FROM, $from_file)
   or die "Can't open from_file: $!\n";
   }
else { printf "No from_file specified!\n"; }

while ($line = <FROM>) {
   if ($line =~ /^#/) {  # Starts with a '#'
      if ($line !~ /^#(Time|EndData)/) {
         chop($line);  # Trim trailing newline
         push(@ntm_hdr, $line);
         }
      }
   else {  # First data record
      close(FROM);
      last;
      }
   }

$to_hdr = shift;  # Second item from command line
if ($to_hdr !~ /(\.|\/)$/) {
   $to_hdr = $to_hdr . ".";
   }
printf "\nto_hdr=$to_hdr\n";

if ($from_file =~ /(\.\w+)$/) {
   $to_trailer = $1;
   printf "to_trailer=$to_trailer\n";
   }

my $day = 6 ; my $hour = 0;  # Sat 00
my $offset = shift;
if ($offset ne '') {
   if ($offset =~ /(^\d)T(\d+)/) {
     $day = $1;  $hour = $2;
     }
   else {
      print "$offset not of form dTdd <<<\n";
      exit;
      }
   }
printf "day=%d, hour=%d\n", $day, $hour;

my $c_time;  # Collection local time (seconds from 1 Jan 1970)
my $starting = 1;
my $nbr = 0;

open(FROM, $from_file);
while (<FROM>) { # read  flows file
#   chop;    # trim new line
   if (/^#/) { # starts with a #
      my $hash_rec = $_;  # Save input line
      if (s/^#Time: //) {
	 if (/(^\d+):(\d+):(\d+) +\w+ +(\d+) +(\w+) +(\d+) +/) {
            #    HH    MM   SS    day    dd    mm     yyyy
	    $c_time = timelocal($3,$2,$1, $4,$::month{$5}-1,$6-1900);
            my @td = localtime($c_time);  # Array of time info
#            my @td = gmtime($c_time);  # Convert to GMT
#            printf "TTT = $td[2]:$td[1], wday=$td[6]\n";

            if ($starting ||  # First sample in from_file
                  ($td[6] == $day && $td[2] == $hour && $td[1] == 0)) {  # Sat 00:00
               start_file($td[2],$td[1],$td[0], $6 % 100,$::month{$5},$4);
  	       $starting = 0;
	       }
	    print TO $hash_rec;
	    }
	 } 
      elsif (/^#EndData: /) {
         print TO $hash_rec;
         }
      next;  # Ignore other # records (e.g. #Ruleset)
      }

   print TO $_;  # Copy data records to output file

#   ++ $nbr;
#   last if $nbr == 2000;
   }

#--------------------------------------------------------------------------

sub start_file {
   my ($sth,$stm,$sts, $yy,$mm,$dd) = @_;  # Arguments

   my $to_file = sprintf ">%s%02d%02d%02d%s",
      $to_hdr, $yy, $mm, $dd, $to_trailer;

   open(TO, $to_file);
   foreach my $rec (@ntm_hdr) {
      print TO $rec, "\n";
      }
   printf TO
      "##local_week.pl  %s %s  starting at %02d:%02d:%02d GMT\n",
      $from_file, $to_file, $sth, $stm, $sts;
   }

sub modify_date {
   my ($date, $incr) = @_;  # Arguments

   $date =~ /(\d\d)(\d\d)\.(\d\d)(\d\d)\"/;
   my $m = $1;  my $d = $2;  my $h = $3;  my $n = $4;
      $h += $incr;
   if ($incr > 0 ) {
      if ($h > 24) {
         $h -= 24;  ++$d;
         }
      }
   else {  # Decrement
      if ($h < 0) {
         $h += 24;  --$d;
         }
      }
   return  sprintf "\"%02d%02d.%02d%02d\"", $m,$d, $h,$n;
   }

