#! /usr/local/bin/perl -w

# plt.dns-root-ucsd.pl: called by plot_dr to do dns response plots
#
# Nevil Brownlee, CAIDA / University of Auckland
#
# 1545, Fri 19 Jan 2001

#  ./plot_dr.pl -d meter -m sdsc -r dns-root -s 20000819 -n 7 -v
#  ./plot_dr.pl -d meter -m sdsc -r dns-root -s 20001029 -n 3 -v -p
#  ./plot_dr.pl -d d/meter -m sdsc -r dns-root-n -s 20001118 -n 7 -v
#  ./plot_dr.pl -d d/meter -m ucsd -r dns-root-ucsd -s 20010127 -n 4 -v
$^W = 1;  # Warnings on
use strict;
#use Symbol;
use Time::Local;

my $home = "/home/nevil";

my $v_dark_c = 7;  # d brown
my $medium_c = 1;  # red
my $other_c  = 5;  # magenta (about the same grey as l green)
my (%root_c, %gtld_c, $dark_c, $light_c);

my $mpn_y2_tics =  "\"A\" 0.75, \"B\" 1.75," .
         " \"C\" 2.75, \"D\" 3.75, \"E\" 4.75," .
         " \"F\" 5.75, \"G\" 6.75, \"H\" 7.75," .
         " \"I\" 8.75, \"J\" 9.75, \"K\" 10.75," .
         " \"L\" 11.75, \"M\" 12.75";

#my $mpn_y2_tics =  "\"A\" 0.75, \"\" 1.5, \"B\" 1.75,  \"\" 2.5," .
#   " \"C\" 2.75, \"\" 3.5, \"D\" 3.75, \"\" 4.5, \"E\" 4.75, \"\" 5.5," .
#   " \"F\" 5.75, \"\" 6.5, \"G\" 6.75, \"\" 5.5, \"H\" 7.75, \"\" 8.5," .
#   " \"I\" 8.75, \"\" 9.5, \"J\" 9.75, \"\" 10.5, \"K\" 10.75, \"\" 11.5," .
#   " \"L\" 11.75, \"\" 12.5, \"M\" 12.75, \"\" 13.5";

my $root_loc = "\"NSI, Va\" 0.75, \"ISI, Ca\" 1.75," .
         " \"PSI, Va\" 2.75, \"UMD, Md\" 3.75, \"Ames, Ca\" 4.75," .
         " \"PAIX, Ca\" 5.75, \"DoD, Va\" 6.75, \"ABRL, Md\" 7.75," .
         " \"Stockholm\" 8.75, \"NSI, Va\" 9.75, \"London\" 10.75," .
         " \"ISI, Ca\" 11.75, \"Tokyo\" 12.75";

my $gtld_loc = "\"NSI\" 0.75, \"JPNIC\" 1.75," .
         " \"AOL\" 2.75, \"Verisign\" 3.75, \"Netscape\" 4.75," .
         " \"Palo Alto\" 5.75, \"NSI\" 6.75, \"----\" 7.75," .
         " \"Stockholm\" 8.75, \"JPNIC\" 9.75, \"London\" 10.75," .
         " \"----\" 11.75, \"Hong Kong\" 12.75";


my $xsf = 1;  my $ysf = 1;
my $x_t_label = +28;

#my $xsf = 0.8;  my $ysf = 0.8;
#my $x_t_label = +18;
#my $xsf = 0.80;  my $ysf = 0.75;
#my $x_t_label = +22;

my ($base_dir, $out_prefix,
   $data_in, $place_time, $site, $tz,
   $start_day, $n_days, $anon, $logo_reqd, $bl_wh, $eps_plot)
   = @ARGV;  # Args from command line

print "plt_dns-root-ucsd ..\n" . 
   " base_dir=$base_dir, out_prefix=$out_prefix,\n" .
   " data_in=$data_in, place_time=$place_time,\n" .
   " site=$site, tz=$tz,\n" .
   " start_day=$start_day, n_days=$n_days,\n" .
   " anon=$anon, logo_reqd=$logo_reqd, bl_wh=$bl_wh, eps_plot=$eps_plot\n";

my ($nstats,$ndistribs,$ncols, $utc_in,$time_label);
get_dist_params($data_in, \$nstats,\$ndistribs,\$ncols);
my $n_servers = $ncols/2;
print "nstats=$nstats, ndistribs=$ndistribs, ncols=$ncols, n_servers=$n_servers\n";
if ($utc_in) {
   $time_label = 'Time: ddThh (UTC)';
   }
else {
   $time_label = "Local Time$tz";
   }
if ($bl_wh) {  # Just use dark_c for strip charts
   $dark_c   = -1;  # black
   $light_c  = 0;  # grey
   %root_c = (
      "A"  =>  $dark_c,  "B"  =>  $dark_c,
      "C"  =>  $dark_c,  "D"  =>  $dark_c,  "E"  =>  $dark_c,
      "F"  =>  $dark_c,  "G"  =>  $dark_c,  "H"  =>  $dark_c,
      "I"  =>  $dark_c,  "J"  =>  $dark_c,  "K"  =>  $dark_c,
      "L"  =>  $dark_c,  "M"  =>  $dark_c
      );
   %gtld_c = (
      "A"  =>  $dark_c,  "B"  =>  $dark_c,
      "C"  =>  $dark_c,  "D"  =>  $dark_c,  "E"  =>  $dark_c,
      "F"  =>  $dark_c,  "G"  =>  $dark_c,  "H"  =>  $dark_c,
      "I"  =>  $dark_c,  "J"  =>  $dark_c,  "K"  =>  $dark_c,
      "L"  =>  $dark_c,  "M"  =>  $dark_c
      );
   }
else {  # Use colours for strip charts
   $dark_c   = 3;  # blue
   $light_c  = 2;  # l green
   %root_c = (
      "A"  =>  $medium_c,  "B"  =>  $dark_c,
      "C"  =>  $medium_c,  "D"  =>  $medium_c,  "E"  =>  $dark_c,
      "F"  =>  $dark_c,    "G"  =>  $medium_c,  "H"  =>  $medium_c,
      "I"  =>  $light_c,   "J"  =>  $medium_c,  "K"  =>  $light_c,
      "L"  =>  $dark_c,    "M"  =>  $light_c
      );
   %gtld_c = (
      "A"  =>  $medium_c,  "B"  =>  $dark_c,
      "C"  =>  $v_dark_c,  "D"  =>  $dark_c,    "E"  =>  $dark_c,
      "F"  =>  $dark_c,    "G"  =>  $medium_c,  "H"  =>  $light_c,
      "I"  =>  $light_c,   "J"  =>  $light_c,  "K"  =>  $light_c,
      "L"  =>  $dark_c,    "M"  =>  $light_c
      );
   }

chdir $base_dir || die "Can't chdir to $base_dir: $!";

# Set globals for plot routines

my ($xstart, $fin_day, $xfinish);
my ($ylab_x,$logo_x,$plt_name_x, $right);

if ($n_days == 3) {  # Special for F GTLD
   $xstart = g_fmt_date($start_day, +12*60);
   $fin_day = modify_date($start_day, $n_days);
   $xfinish = g_fmt_date($fin_day, -12*60);
   $ylab_x = g_fmt_date($start_day, +12*60 - 40);
   $logo_x = g_fmt_date($start_day, +12*60 + 90);
   $plt_name_x = g_fmt_date($start_day, +3*60);
   $right  = g_fmt_date($fin_day, -12*60 - 30);
   }
else {
   $xstart = g_fmt_date($start_day);
   $fin_day = modify_date($start_day, $n_days);
   $xfinish = g_fmt_date($fin_day);
   if ($n_days == 1) {
      $ylab_x = g_fmt_date($start_day, -20);
      $logo_x = g_fmt_date($start_day);
#      $logo_x = g_fmt_date($start_day, +30);
      $plt_name_x = g_fmt_date($start_day, +30);
      $right  = g_fmt_date($fin_day, -30);
      }
   else {
      $ylab_x = g_fmt_date($start_day, -1.5*60);
#      $ylab_x = g_fmt_date($start_day, -2*60);
      $logo_x = g_fmt_date($start_day, +3*60);
#      $logo_x = g_fmt_date($start_day, +3*60);
      $plt_name_x = g_fmt_date($start_day, +3*60);
      $right  = g_fmt_date($fin_day, -3*60);
      }
   }

print "@@@ start_day=$start_day, plt_name_x=$plt_name_x\n";
print "fin_day=$fin_day, logo_x=$logo_x, ylab_x=$ylab_x, right=$right\n";

#  Evi's plots for F GTLD (SIGCOMM paper)
#plot_dns($out_prefix, 14, $data_in, $site, $tz);

#  FM + AG response plots (SIGCOMM paper)
#plot_dns($out_prefix, 11, $data_in, $site, $tz);

#plot_dns($out_prefix, 21, $data_in, $site, $tz);
#plot_dns($out_prefix, 22, $data_in, $site, $tz);
#plot_dns($out_prefix, 23, $data_in, $site, $tz);
#plot_dns($out_prefix, 24, $data_in, $site, $tz);
#plot_dns($out_prefix, 25, $data_in, $site, $tz);

plot_dns($out_prefix, 31, "$data_in.ss", $site, $tz);
plot_dns($out_prefix, 32, "$data_in.ss", $site, $tz);
plot_dns($out_prefix, 33, "$data_in.ss", $site, $tz);
plot_dns($out_prefix, 34, "$data_in.ss", $site, $tz);
plot_dns($out_prefix, 35, "$data_in.ss", $site, $tz);
plot_dns($out_prefix, 36, "$data_in.ss", $site, $tz);

plot_dns($out_prefix, 41, "$data_in", $site, $tz);
plot_dns($out_prefix, 42, "$data_in", $site, $tz);
plot_dns($out_prefix, 43, "$data_in", $site, $tz);

#plot_dns($out_prefix, 51, "$data_in.3q", $site, $tz);


exit;


sub plot_dns {
   my ($out_prefix, $which, $dataname, 
      $site, $tz) = @_;  # Arguments

   my ($tempname, $plotname);
   $out_prefix =~ /^([^\/]+)/;
   my $ruleset = $1; 
   my $a_date;

   if ($start_day == 20010113 && $n_days == 7) {
      $a_date = ', Jan 13-20 2001';
      }
   elsif ($start_day == 20001127 && $n_days == 4) {
      $a_date = ', Jan 27-31 2001';
      }
   elsif ($start_day == 20001130 && $n_days == 1) {
      $a_date = ', Nov 30 2000';
      }
   else {
      $a_date = " $place_time";
      }

   if ($which == 11) {  # F&M over A&G, response time only
      $plotname = 'fmag';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot2(\*PLOT, $site, $tz,
         "Root nameserver performance $place_time",
         "F root (Palo Alto)",$dataname, 7, $root_c{"F"},
         "M root (Tokyo)",    $dataname,14, $root_c{"M"},
         "A root (NSI, Va)",  $dataname,2,  $root_c{"A"},
         "G root (DoD, Va)",  $dataname,8,  $root_c{"G"});
#         "F root (Palo Alto)",$dataname, 7, $root_c{"F"},
#         "M root (Tokyo)",    $dataname,14, $root_c{"M"},
#         "A root (NSI, Va)",  $dataname,2,  $root_c{"A"},
#         "G root (DoD, Va)",  $dataname,8,  $root_c{"G"});
      close (PLOT);
      }
   elsif ($which == 14) {
      $plotname = 'f-gt';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot3(\*PLOT, $site, $tz, 
         "gTLD nameserver performance $place_time",
#         "FG 53 53", $dataname, 15,30, $gtld_c{"M"},
         "F (Palo Alto)",  $dataname, 16,31, 79,24, $gtld_c{"F"});
      close (PLOT);
      }

   elsif ($which == 21) {
      $plotname = 'root.af';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $site, $tz,
         "Root nameserver performance $place_time",
         'A (NSI, Va)', $dataname,2,28, $root_c{'A'},
         'F (Palo Alto)', $dataname, 7,33, $root_c{'F'});
      close (PLOT);
      }
   elsif ($which == 22) {
      $plotname = 'gtld.df';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $site, $tz,
         "gTLD nameserver performance $place_time",
         'D (Verisign)', $dataname,18,44, $gtld_c{'D'},
         'F (Palo Alto)', $dataname, 20,46, $gtld_c{'F'});
      close (PLOT);
      }
   elsif ($which == 23) {
      $plotname = 'gtld.ag';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $site, $tz,
         "gTLD nameserver performance $place_time",
         "A (NSI, Va)", $dataname,15,41, $gtld_c{'A'},
         "G (NSI, Va)", $dataname, 21,47, $gtld_c{'G'});
      close (PLOT);
      }
   elsif ($which == 24) {
      $plotname = 'gt-ro.f';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $site, $tz,
         "gTLD & root server performance $place_time",
         'F gTLD', $dataname, 20,46, $gtld_c{'F'},
         'F root', $dataname, 7,33, $light_c);
      close (PLOT);
      }
   elsif ($which == 25) {
      $plotname = 'gt-ro.a';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $site, $tz,
         "gTLD & root server performance $place_time",
         "A gTLD", $dataname,15,41, $root_c{'A'},
         'A root', $dataname,2,28, $light_c);
      close (PLOT);
      }
   elsif ($which == 31) {
      $plotname = 'root.count';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot_n(\*PLOT, $site, $tz,
         "Root Requests$a_date, scale 0-200 packets",
#         "Root requests (0..200) $place_time",
         $dataname, 0, 2, $root_loc);
      close (PLOT);
      }
   elsif ($which == 32) {
      $plotname = 'gtld.count';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot_n(\*PLOT, $site, $tz, 
         "gTLD Requests$a_date, scale 0-200 packets",
#         "gTLD requests (0..200) $place_time",
         $dataname, $n_servers, 2, $gtld_loc);
      close (PLOT);
      }
   elsif ($which == 33) {
      $plotname = 'root.losspc';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot_n(\*PLOT, $site, $tz, 
         "Root Percent Loss$a_date, scale 0-100 %",
#         "Root loss % (0..100) $place_time",
         $dataname, 0, 1, $root_loc);
      close (PLOT);
      }
   elsif ($which == 34) {
      $plotname = 'gtld.losspc';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot_n(\*PLOT, $site, $tz,
         "gTLD Percent Loss$a_date, scale 0-100 %",
#         "gTLD loss % (0..100) $place_time",
         $dataname, $n_servers, 1, $gtld_loc);
      close (PLOT);
      }
   elsif ($which == 35) {
      $plotname = 'root.rtt';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot_n(\*PLOT, $site, $tz, 
         "Root Response Time$a_date, scale 0-300 ms",
#         "Root response time (0..300 ms) $place_time",
         $dataname, 0, 0, $root_loc);
      close (PLOT);
      }
   elsif ($which == 36) {
      $plotname = 'gtld.rtt';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot_n(\*PLOT, $site, $tz,
         "gTLD Response Time$a_date, scale 0-200 ms",
#         "gTLD response time (0..200 ms) $place_time",
         $dataname, $n_servers, 0, $gtld_loc);
      close (PLOT);
      }
   elsif ($which == 41) {
      $plotname = 'root.af.4plot';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_plot_4(\*PLOT, $site, $tz,
         "Root Nameserver Performance$a_date",
#         "Root Nameserver Performance $place_time",
         $dataname, 'data-rate.plt', 0, # 0=median, 1=min, 2=max
         'A (NSI, Va)',   0, 1, $light_c,
         'F (Palo Alto)', 0, 6, $dark_c);
      close (PLOT);
      }
   elsif ($which == 42) {
      $plotname = 'gtld.af.4plot';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_plot_4(\*PLOT, $site, $tz,
         "gTLD Nameserver Performance$a_date",
#         "gTLD Nameserver Performance $place_time",
         $dataname, 'data-rate.plt', 0, # 0=median, 1=min, 2=max
         'A (NSI)',   $n_servers, 1, $light_c,
         'F (Palo Alto)', $n_servers, 6, $dark_c);
      close (PLOT);
      }
   elsif ($which == 43) {
      $plotname = 'both.f.4plot';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_plot_4(\*PLOT, $site, $tz,
         "Nameserver Performance$a_date",
#         "Nameserver Performance $place_time",
         $dataname, 'data-rate.plt', 0, # 0=median, 1=min, 2=max
         'F gTLD', $n_servers, 6, $dark_c,
         'F root', 0, 6, $light_c);
#         'F gTLD', $n_servers, 6, $light_c,
#         'F root', 0, 6, $dark_c);
      close (PLOT);
      }
   elsif ($which == 51) {
      $plotname = 'af.3q';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot_4x3(\*PLOT, $site, $tz,
         "Nameserver response time $place_time", $dataname,
         'F root', 6, $dark_c);  # F root
      close (PLOT);          # 50%, 25%, 75%
      }
#   elsif ($which == 52) {
#      $plotname = 'gtld.af.3q';
#      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
#      put_multiplot5(\*PLOT, $site, $tz,
#         "gTLD nameserver performance $place_time",
#         'A (NSI, Va)', $dataname, 18,44,70, $gtld_c{'A'},
#         'F (Palo Alto)', $dataname, 20,46,72, $gtld_c{'F'});
#      close (PLOT);
#      }
   else { return; }

   if ($eps_plot) {
      printf ">>> gnuplot $tempname > $out_prefix.$plotname.eps\n";
      system("gnuplot $tempname > $out_prefix.$plotname.eps");
      }
   else {
      printf ">>> gnuplot $tempname > $out_prefix.$plotname.png\n";
      system("gnuplot $tempname > $out_prefix.$plotname.png");
      }
   #   unlink "$tempname";
   }



sub get_dist_params {
   my ($from_file, $st_ref,$dist_ref,$cols_ref) = @_;  # Arguments

   if (defined($from_file)) {
      open(FROM, $from_file)
      or die "Can't open from_file: $!\n";
      }
 
   my (@attribs, @lo, @hi, @nvalues);
   my $n = 0;  my $mycols = 1;

   while (<FROM>) { # read  flows file
      chop;  # Trim new line
      if (/^#/) {  # Starts with a #
         if (/^##NeTraMet /) {
            if (/\(UTC\)/) {  $utc_in = 1; }
	     print " ==== utc_in = 1\n";
            }
         elsif (/^##plt_stat /) {
            my @items = split(/ +/, $_);
            foreach my $item (@items) {
               if ($item =~ /(.+)=(\d+)\.\.(\d+)/) {
                  $attribs[$n] = $1;
	          $lo[$n] = $2;  $hi[$n] = $3;
                  $nvalues[$n] = $3-$2+1;
	          $mycols *= $nvalues[$n];
	          ++$n;
                  }
               elsif ($item =~ /stats=(\d+)/) {
                  $$st_ref = $1;
                  }
               elsif ($item =~ /distribs=(\d+)/) {
                  $$dist_ref = $1;  $mycols *= $ndistribs;
                  }
               elsif ($item =~ /cols=(\d+)/) {
                  $$cols_ref = $1;
                  }
               }
            close(FROM);
            return;
	    }
	 } 
      next;
      }
   }

sub put_plot_4 {
   my ($FILE, $site, $tz, $title, $dnsdata, $ratedata, $ratestat,
      $dnskey,  $doff, $dt,$dc, # key, offset rtt col, colour (plotted first)
      $pingkey, $poff, $pt,$pc) = @_;  # Arguments
            # Median, loss %, total count, ToLostPDUs
   my ($r1,$r2);

   my $s_ysf = $ysf;
   my $s_x_t_label = $x_t_label;
   if ($ysf != 1) { $ysf += 0.05; $x_t_label += 5; };

   $dt += 1 + $doff;  $pt += 1 + $poff;  # Add offset (root/gTLD/Nominum)
  
   print $FILE "set multiplot\n";
   print $FILE "set title \"$title\"\n";
   printf $FILE "set origin %f, %f\n", 0*$xsf, 0.67*$ysf;
   printf $FILE "set size %f, %f\n", 0.982*$xsf, 0.32*$ysf;
   print $FILE "set yrange [0:249]\n";
   print $FILE "set ytics 50\n";
#   print $FILE "set label 1 \"ms\" at $ylab_x, 245 right\n";
   print $FILE "set label 1 \"ms\" at $ylab_x, 235 right\n";
   print $FILE "set label 2 \"DNS response time\" at $plt_name_x, 220\n";
   print $FILE "plot \"$dnsdata\" u 1:$dt t \"$dnskey\" w li $dc";
   if ($pingkey ne '') 
      { print $FILE ", \"$dnsdata\" u 1:$pt t \"$pingkey\" w li $pc"; }
   print $FILE "\n";

   print $FILE "set title\n";
   printf $FILE "set origin %f, %f\n", 0.012*$xsf, 0.475*$ysf;
   printf $FILE "set size %f, %f\n", 0.97*$xsf, 0.215*$ysf;
   print $FILE "set yrange [0:39.5]\n";
   print $FILE "set ytics 10\n";
#   print $FILE "set label 1 \"%\" at $ylab_x, 42 right\n";
   print $FILE "set label 1 \"%\" at $ylab_x, 38 right\n";
   print $FILE "set label 2 \"Packet loss\" at $plt_name_x, 33\n";
   print $FILE "set nokey\n";

   $r1 = $dt + $ncols;  # Loss %
   $r2 = $pt + $ncols;
   print $FILE "plot \"$dnsdata\" u 1:$r1 t \"$dnskey loss %\" w li $dc";
   if ($pingkey ne '') 
      { print $FILE ", \"$dnsdata\" u 1:$r2 t \"$pingkey loss %\" w li $pc"; }
   print $FILE "\n";

   printf $FILE "set origin %f, %f\n", 0.003*$xsf, 0.28*$ysf;
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.218*$ysf;
#   print $FILE "set origin 0, 0.28\n";
#   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.22*$ysf;
   print $FILE "set yrange [0:199]\n";
   print $FILE "set ytics 50\n";
#   print $FILE "set label 1 \"reqs\" at $ylab_x, 200 right\n";
   print $FILE "set label 1 \"reqs\" at $ylab_x, 190 right\n";
   print $FILE "set label 2 \"DNS requests\" at $plt_name_x, 165\n";
   print $FILE "set nokey\n";

   $r1 = $dt + $ncols*2;  # Total requests
   $r2 = $pt + $ncols*2;
   print $FILE "plot \"$dnsdata\" u 1:$r1 t \"$dnskey loss %\" w li $dc";
   if ($pingkey ne '') 
      { print $FILE ", \"$dnsdata\" u 1:$r2 t \"$pingkey loss %\" w li $pc"; }
   print $FILE "\n";

   print $FILE "set label 1\n";
   printf $FILE "set origin %f, %f\n", 0.013*$xsf, 0.05*$ysf;
   printf $FILE "set size %f, %f\n", 0.97*$xsf, 0.24*$ysf;
   print $FILE "set yrange [0:19.9]\n";
   print $FILE "set ytics 5\n";
#   print $FILE "set label 1 \"Mbps\" at $ylab_x, 21 right\n";
   print $FILE "set label 1 \"Mbps\" at $ylab_x, 19.5 right\n";
   print $FILE "set label 2 \"10s data rate\" at $plt_name_x, 4\n";
#   print $FILE "set label 2 \"10s data rate\" at $plt_name_x, 16.5\n";

   my $logo = $logo_reqd ?
      'Nevil Brownlee, U Auckland / CAIDA\\nCoral NeTraMet,'
      : '';
   print $FILE
#      "set label 3 \"$logo median of 5-min samples\" at \"$logo_x.0000\", -10.5\n";
      "set label 3 \"$logo Median of 5-min samples\" at \"$logo_x.0000\", -8\n";
   print $FILE "set xlabel \"$time_label\" $x_t_label\n";
   print $FILE "set key bottom right\n";
#   print $FILE "set key\n";

print "about to open $ratedata\n";
   my ($d_stats, $d_distribs, $d_cols);
   get_dist_params($ratedata, \$d_stats,\$d_distribs,\$d_cols);
   print "d_stats=$d_stats, d_distribs=$d_distribs, d_cols=$d_cols\n";

   $r1 = 1 + 1 + $ratestat*$d_cols;
   $r2 = 1 + 2 + $ratestat*$d_cols;
print "----- r_to=$r1, r_from=$r2\n";
   print $FILE "plot \"$ratedata\" u 1:$r1 t \"outbound\" w li $dc";
   if ($pingkey ne '') 
      { print $FILE ", \"$ratedata\" u 1:$r2 t \"inbound\" w li $pc"; }
   print $FILE "\n";

   print $FILE "set nomultiplot\n";
   $ysf = $s_ysf;  $x_t_label = $s_x_t_label;
   }

sub put_multiplot_4x3 {
   my ($FILE, $site, $tz, $title, $dnsdata,
      $key1, $d11,$d12,$d13,  $key2 ,$d21,$d22,$d23,
      $key3, $d31,$d32,$d33,  $key4 ,$d41,$d42,$d43
      ) = @_;  # Arguments

   print $FILE "set multiplot\n";
   print $FILE "set title \"$title\"\n";
   print $FILE "set origin 0, 0.72\n";
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.22*$ysf;
   print $FILE "set yrange [0:179]\n";
   print $FILE "set ytics 50\n";
   print $FILE "set label 1 \"ms\" at $ylab_x, 175 right\n";
   print $FILE "set label 2 \"$key1\" at $plt_name_x, 155\n";
   print $FILE "set nokey\n";
   print $FILE "plot \"$dnsdata\" u 1:$d13 t \"75 %\" w li $dark_c," .
      " \"\" u 1:$d12 t \"median\" w li $medium_c," .
      " \"\" u 1:$d11 t \"25 %\" w li $light_c\n";

   print $FILE "set title\n";
   print $FILE "set origin 0, 0.50\n";
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.22*$ysf;
   print $FILE "set label 2 \"$key2\" at $plt_name_x, 150\n";
   print $FILE "set nokey\n";
   print $FILE "plot \"$dnsdata\" u 1:$d23 t \"75 %\" w li $dark_c," .
      " \"\" u 1:$d22 t \"median\" w li $medium_c," .
      " \"\" u 1:$d21 t \"25 %\" w li $light_c\n";

   print $FILE "set title\n";
   print $FILE "set origin 0, 0.28\n";
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.22*$ysf;
   print $FILE "set label 2 \"$key3\" at $plt_name_x, 150\n";
   print $FILE "set nokey\n";
   print $FILE "plot \"$dnsdata\" u 1:$d33 t \"75 %\" w li $dark_c," .
      " \"\" u 1:$d32 t \"median\" w li $medium_c," .
      " \"\" u 1:$d31 t \"25 %\" w li $light_c\n";

   my $logo = $logo_reqd ? 
      'Nevil Brownlee, U Auckland / CAIDA\\nCoral NeTraMet,'
      : '';
   print $FILE
      "set label 3 \"$logo quartiles of 5-min samples\" at \"$logo_x.0000\", -18.5\n";
   print $FILE "set xlabel \"$time_label\" $x_t_label\n";

   print $FILE "set title\n";
   print $FILE "set origin 0, 0.06\n";
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.22*$ysf;
   print $FILE "set label 2 \"$key4\" at $plt_name_x, 150\n";
   print $FILE "set key below\n";
   print $FILE "plot \"$dnsdata\" u 1:$d43 t \"75 %\" w li $dark_c," .
      " \"\" u 1:$d42 t \"median\" w li $medium_c," .
      " \"\" u 1:$d41 t \"25 %\" w li $light_c\n";

   print $FILE "set nomultiplot\n";
   }

sub put_multiplot {
   my ($FILE, $site, $tz, $title,
      $dnskey,$dnsdata, $dt,$dl, $dc,
      $pingkey,$pingdata, $pt,$pl, $pc) = @_;  # Arguments

   print $FILE "set multiplot\n";
   print $FILE "set title \"$title\"\n";
   print $FILE "set origin 0, 0.3\n";
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.625*$ysf;
   print $FILE "set yrange [0:249]\n";
   print $FILE "set ytics 100\n";
   print $FILE "set label 1 \"ms\" at $ylab_x, 245 right\n";
   print $FILE "set label 2 \"DNS response time\" at $plt_name_x, 230\n";
   print $FILE "plot \"$dnsdata\" u 1:$dt t \"$dnskey\" w li $dc";
   if ($pingdata ne '') 
      { print $FILE ", \"$pingdata\" u 1:$pt t \"$pingkey\" w li $pc"; }
   print $FILE "\n";

   my $logo = $logo_reqd ?
      'Nevil Brownlee, U Auckland / CAIDA\\nCoral NeTraMet,'
      : '';
   print $FILE
      "set label 3 \"$logo Median of 5-min samples\" at \"$logo_x.0000\", -18.5\n";

   print $FILE "set label 1\n";
   print $FILE "set title\n";
   print $FILE "set origin 0.013, 0.06\n";
   printf $FILE "set size %f, %f\n", 0.97*$xsf, 0.34*$ysf;
   print $FILE "set yrange [0:49.5]\n";
   print $FILE "set ytics 10\n";
   print $FILE "set label 1 \"%\" at $ylab_x, 47 right\n";
   print $FILE "set label 2 \"Packet loss\" at $plt_name_x, 42\n";
   print $FILE "set xlabel \"Local Time$tz\" $x_t_label, +0\n";
   print $FILE "set nokey\n";
   print $FILE "plot \"$dnsdata\" u 1:$dl t \"$dnskey loss %\" w li $dc";
   if ($pingdata ne '') 
      { print $FILE ", \"$pingdata\" u 1:$pl t \"$pingkey loss %\" w li $pc"; }
   print $FILE "\n";

   print $FILE "set nomultiplot\n";
   }

sub put_multiplot2 {
   my ($FILE, $site, $tz, $title,
      $dnskey,$dnsdata,$dt,$dc,
      $pingkey,$pingdata,$pt,$pc,
      $dnskey2,$dnsdata2,$dt2,$dc2,
      $pingkey2,$pingdata2,$pt2,$pc2) = @_;  # Arguments

   print $FILE "set multiplot\n";
   print $FILE "set title \"$title\"\n";
   print $FILE "set origin 0, 0.52\n";
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.47*$ysf;
   print $FILE "set yrange [0:680]\n";
   print $FILE "set ytics 100\n";
   print $FILE "set label 1 \"ms\" at $ylab_x, 670 right\n";
   print $FILE "set label 2 \"DNS response time (ms)\" at \"$logo_x.0000\", 640\n";
   print $FILE "plot \"$dnsdata\" u 1:$dt t \"$dnskey\" w li $dc";
   if ($pingdata ne '') 
      { print $FILE ", \"$pingdata\" u 1:$pt t \"$pingkey\" w li $pc"; }
   print $FILE "\n";

   my $logo = $logo_reqd ?
      'Nevil Brownlee, U Auckland / CAIDA\\nCoral NeTraMet,'
      : '';
   print $FILE
      "set label 3 \"$logo Median of 5-min samples\" at \"$logo_x.0000\", -110\n";
   print $FILE "set label 1\n";
   print $FILE "set title\n";
   print $FILE "set origin 0.0, 0.07\n";
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.45*$ysf;
   print $FILE "set xlabel \"Local Time$tz\" $x_t_label, +0\n";
   print $FILE "plot \"$dnsdata\" u 1:$dt2 t \"$dnskey2\" w li $dc2";
   if ($pingdata ne '') 
      { print $FILE ", \"$pingdata\" u 1:$pt2 t \"$pingkey2\" w li $pc2"; }
   print $FILE "\n";

   print $FILE "set nomultiplot\n";
   }

sub put_multiplot3 {
   my ($FILE, $site, $tz, $title,
      $dnskey,$dnsdata, $dt,$dl, $dtu,$dlu, $dc,
      $pingkey,$pingdata, $pt,$pl, $ptu,$plu, $pc) = @_;  # Arguments

   print $FILE "set multiplot\n";
   print $FILE "set title \"$title\"\n";
   print $FILE "set origin 0, 0.575\n";
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.425*$ysf;
   print $FILE "set yrange [0:$dtu]\n";
   print $FILE "set ytics 10\n";
   my $h6 = 6*60*60;  # Seconds in 6 hours
   print $FILE "set xtics $h6 \n";
   print $FILE "set mxtics 2 \n";
   print $FILE "set label 1 \"ms\" at $ylab_x, $dtu right\n";
   print $FILE "set label 2 \"DNS response time (ms)\" at \"$logo_x.0000\", $dtu*0.9\n";
   print $FILE "plot \"$dnsdata\" u 1:$dt t \"$dnskey\" w li $dc";
   if ($pingdata ne '') 
      { print $FILE ", \"$pingdata\" u 1:$pt t \"$pingkey\" w li $pc"; }
   print $FILE "\n";

   my $logo = $logo_reqd ?
      'Nevil Brownlee, U Auckland / CAIDA\\nCoral NeTraMet,'
      : '';
   print $FILE
      "set label 3 \"$logo Median of 5-min samples\" at \"$logo_x.0000\", -$dlu*0.34\n";

   print $FILE "set label 1\n";
   print $FILE "set title\n";
   print $FILE "set origin 0., 0.24\n";
   printf $FILE "set size %f, %f\n", 0.98*$xsf, 0.34*$ysf;
   print $FILE "set yrange [-2:$dlu]\n";
   print $FILE "set ytics 5\n";
   print $FILE "set label 1 \"%\" at $ylab_x, $dlu right\n";
   print $FILE "set label 2 \"Packet loss (%)\" at \"$logo_x.0000\", $dlu*0.85\n";
   print $FILE "set xlabel \"Local Time$tz\" $x_t_label, +0\n";
   print $FILE "set nokey\n";
   print $FILE "plot \"$dnsdata\" u 1:$dl t \"$dnskey loss %\" w li $dc";
   if ($pingdata ne '') 
      { print $FILE ", \"$pingdata\" u 1:$pl t \"$pingkey loss %\" w li $pc"; }
   print $FILE "\n";

   print $FILE "set nomultiplot\n";
   }

sub put_multiplot_n {
   my ($FILE, $site, $tz, $title,
      $dataname, $origin, $stat, $names) = @_;  # Arguments

      print $FILE "set multiplot\n";
      print $FILE "set origin -0.05, 0\n";
      printf $FILE "set size %f, %f\n", 1.05*$xsf, 1*$ysf;
#      print $FILE "set origin 0, 0.05\n";
#      printf $FILE "set size %f, %f\n", 1*$xsf, 0.95*$ysf;
      print $FILE "set title \"$title\"\n";
      print $FILE "set xlabel \"$time_label\" $x_t_label, +0\n";
#      if ($logo_reqd) {
#         print PLOT
#            "set label 3 \"Coral NeTraMet, 5-min samples\\nNevil Brownlee," .
#            " U Auckland / CAIDA\" at \"$logo_x.0000\", -0.47\n";
#         }
      my $logo = $logo_reqd ?
         'Nevil Brownlee, U Auckland / CAIDA\\nCoral NeTraMet,'
         : '';
      print $FILE
         "set label 3 \"$logo Median of 5-min samples\" at \"$logo_x.0000\", -0.38\n";
      print $FILE "set nokey\n";
      print $FILE "set yrange [0.5:14]\n";
      print $FILE "set ytics nomirror;\n";
      print $FILE "set ytics ($names)\n";
      print $FILE "set y2tics nomirror;\n";
      print $FILE "set y2tics ($mpn_y2_tics)\n";
#      print $FILE "set y2tics (\"A\" 0.75, \"B\" 1.75, \"C\" 2.755," .
#         " \"D\" 3.75, \"E\" 4.75, \"F\" 5.75, \"G\" 6.75, \"H\" 7.75," .
#         " \"I\" 8.75, \"J\" 9.75, \"K\" 10.75, \"L\" 11.75, \"M\" 12.75)\n";

      my @c;
      for (my $x = 1; $x <= $n_servers; ++$x) {
         $c[$x] = 1 + $origin + $x + $stat*$ncols;
         }

      my %sc;
      %sc = $origin == 0 ? %root_c : %gtld_c;
      print $FILE
         "plot \"$dataname\" u 1:$c[1] w st $sc{'A'}" .
         ", \"\" u 1:$c[2] w st $sc{'B'}" .
         ", \"\" u 1:$c[3] w st $sc{'C'}" .
         ", \"\" u 1:$c[4] w st $sc{'D'}" .
         ", \"\" u 1:$c[5] w st $sc{'E'}" .
         ", \"\" u 1:$c[6] w st $sc{'F'}" .
         ", \"\" u 1:$c[7] w st $sc{'G'}" .
         ", \"\" u 1:$c[8] w st $sc{'H'}" .
         ", \"\" u 1:$c[9] w st $sc{'I'}" .
         ", \"\" u 1:$c[10] w st $sc{'J'}" .
         ", \"\" u 1:$c[11] w st $sc{'K'}" .
         ", \"\" u 1:$c[12] w st $sc{'L'}" .
         ", \"\" u 1:$c[13] w st $sc{'M'}" .
         "\n";
      print $FILE "set nomultiplot\n";
   }


sub put_gnu_header {
   my ($FILE, $timefmt, $xstart,$xfinish) = @_;  # Arguments

   if ($eps_plot) {
      print $FILE "set term postscript eps monochrome\n";
      }
   else {
      print $FILE "set term png small color\n";
      }

   print $FILE "set missing \"?\"\n";
   if ($timefmt ne '') {
      print $FILE "set timefmt \"%Y%m%d.%H%M\"\n";
      print $FILE "set xdata time\n";
      print $FILE "set format x \"%a%02dT%H\"\n";  # ISO 8601 uses T (Ken Keys)
      if (defined($xstart)) {
         print PLOT "set xrange [\"$xstart.0000\":\"$xfinish.0000\"]\n";
         }
      }
   }

sub open_tmp {
    my ($FILE, $tempname) = @_;  # Arguments
   if (!open($FILE, ">$tempname") ) {
      printf STDERR "Couldn't open $tempname for write <<<\n";
      exit;
      }
   put_gnu_header($FILE, 't', $xstart,$xfinish);
   }


sub u_seconds {
   my ($t_day) = @_;  # Arguments

   my ($fy, $fm, $fd);
   $t_day =~ /^(\d\d\d\d)(\d\d)(\d\d)/;
   $fy = $1;  $fm = $2;  $fd = $3;
   return timelocal(0, 0, 0, $fd, $fm-1, $fy-1900);
   }

sub modify_date {
   my ($t_day, $d_incr) = @_;  # Arguments

   my $fs = u_seconds($t_day);

   my @fyd = localtime($fs + $d_incr * 24*60*60);  # + hours
   my $r = sprintf "%04d%02d%02d",
      $fyd[5]+1900, $fyd[4]+1, $fyd[3];  # yyyy mm dd
   return $r;
   }

sub g_fmt_date {
   my ($t_day, $h_incr) = @_;  # Arguments (h_incr in minutes)

   my $fs = u_seconds($t_day);

   if (defined($h_incr)) { $fs += $h_incr*60; }

   my @fyd = localtime($fs);
   my $r = sprintf "%04d%02d%02d",
      $fyd[5]+1900, $fyd[4]+1, $fyd[3];  # yyyy mm dd
   return $r;
   }
