#! /usr/local/bin/perl -w

# 2024, Tue 6 Jun 00 (PST), CAIDA
#
# Nevil Brownlee, ITSS Technology Development, The University of Auckland


use strict;

local $::ln10 = 2.3025850929940457;

my %dist = (
   "Trans" => 1,  # Linear
   "Scale" => 0,  # us, Byte
   "Lower" => 1, "Upper" => 1000,
   "Buckets" => 100
   );

for (;;) {
   get_dist_params();
   print "SRL: $dist{Buckets}.PP.0!0 & " .
      "$dist{Trans}.$dist{Scale}.$dist{Lower}!$dist{Upper};\n";
   put_distrib(\%dist);
   print "\n";
   }

exit;


sub put_distrib {
   my($distrib) = @_;  # Arguments

   my ($trans, $scale, $low,$high, $nb, $step);

   my $dist = $distrib;

   $trans = $dist->{Trans};
   $scale =  $dist->{Scale};
   $low = $dist->{Lower}*10**$scale;
   $high = $dist->{Upper}*10**$scale;
   $nb = $dist->{Buckets};

   if ($trans == 1) {
      $step = ($high - $low) / ($nb - 1);
      }
   else {
      $step = (log($high) - log($low)) / (($nb - 1)*$::ln10);
      }
   printf "     Step=%s%.2f\n", ($trans == 1 ? '+' : '*'), $step;

   for (my $j = 0; $j < $nb; $j += 10) {
      printf "%3d ", $j;
      for (my $k = 0; $k != 10; ++$k) {
	 my $m = $j + $k;
         last if $m == $nb;
         my $d = $trans == 1 ? $low + $m*$step :
                  $low * 10**($step*$m);
         printf " %.2f", $d;
         }
      print "\n";
      }
   }

sub get_dist_params{
   my $v;
   print "Enter distribution parameters ..\n";
   print "   Buckets ($dist{Buckets}): ";
   $v = <>;  chop($v);
   if ($v ne '') { $dist{Buckets} = $v; }

   print "   Transform ($dist{Trans}): ";
   $v = <>;  chop($v);
   if ($v ne '') { $dist{Trans} = $v; }

   print "   Scale ($dist{Scale}): ";
   $v = <>;  chop($v);
   if ($v ne '') { $dist{Scale} = $v; }

   print "   Lower ($dist{Lower}): ";
   $v = <>;  chop($v);
   if ($v ne '') { $dist{Lower} = $v; }

   print "   Upper ($dist{Upper}): ";
   $v = <>;  chop($v);
   if ($v ne '') { $dist{Upper} = $v; }
   }
